////////////////////////////////////////////////////////////////////////
/*
     FILE: MVISR.java
  PURPOSE: Implements the CMA MVISR radiometer.
   AUTHOR: Peter Hollemans
     DATE: 2007/03/16
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.instrument;

/**
 * The <code>MVISR</code> is a radiometer for the China
 * Meteorological Administration (CMA) Multispectral Visible and IR
 * Scan Radiometer.
 */
public class MVISR 
  implements Radiometer {

  // Constants
  // ---------

  /** The number of sample value for the MVISR sensor. */
  public static final int SAMPLES = 2048;

  // Variables
  // ---------

  /** The singleton instance of this class. */
  private static MVISR instance = new MVISR();

  ////////////////////////////////////////////////////////////

  private MVISR () {}

  ////////////////////////////////////////////////////////////

  /** Gets the singleton instance of this class. */
  public static MVISR getInstance() { return (instance); }

  ////////////////////////////////////////////////////////////

  public String getName() { return ("mvisr"); }

  ////////////////////////////////////////////////////////////

  public String getLongName() { 

    return ("Multispectral Visible and IR Scan Radiometer");

  } // getLongName

  ////////////////////////////////////////////////////////////

  public int getChannelCount() { return (10); }

  ////////////////////////////////////////////////////////////

  public String getChannelName (int channel) {
    
    switch (channel) {
    case 1: return ("mvisr_ch1");
    case 2: return ("mvisr_ch2");
    case 3: return ("mvisr_ch3");
    case 4: return ("mvisr_ch4");
    case 5: return ("mvisr_ch5");
    case 6: return ("mvisr_ch6");
    case 7: return ("mvisr_ch7");
    case 8: return ("mvisr_ch8");
    case 9: return ("mvisr_ch9");
    case 10: return ("mvisr_ch10");
    default: throw new IllegalArgumentException();
    } // switch

  } // getChannelName

  ////////////////////////////////////////////////////////////

  public boolean isThermal (int channel) {

    switch (channel) {
    case 1: case 2: case 6: case 7: case 8: case 9: case 10: return (false);
    case 3: case 4: case 5: return (true);
    default: throw new IllegalArgumentException();
    } // switch

  } // isThermal

  ////////////////////////////////////////////////////////////

  public int getSampleCount() { return (SAMPLES); }

  ////////////////////////////////////////////////////////////

} // MVISR class

////////////////////////////////////////////////////////////////////////
