////////////////////////////////////////////////////////////////////////
/*
     File: RadiometerDataWriter.java
  PURPOSE: Writes radiometer data to an data sink.
   AUTHOR: Peter Hollemans
     DATE: 2007/03/13
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.instrument;

// Imports
// -------
import java.io.*;
import terrenus.telemetry.*;
import terrenus.instrument.RadiometerCalibrator.CalibrationType;

/**
 * The <code>RadiometerDataWriter</code> interface is for classes
 * that write radiometer data frames to an output data sink.
 */
public interface RadiometerDataWriter {

  ////////////////////////////////////////////////////////////

  /**
   * Writes radiometer channel and location data to a data output
   * sink.
   *
   * @param decoder the stream decoder to read data from.
   * @param radiometer the radiometer instrument to write data for.
   * @param startFrame the starting data frame to write.
   * @param endFrame the ending data frame to write.
   * @param visibleCalType the visible channel calibration type.
   * @param thermalCalType the thermal channel calibration type.
   *
   * @throws IOException if an error occurred reading the
   * telemetry stream or writing radiometer data.
   */
  public void writeData (
    TelemetryDecoder decoder,
    Radiometer radiometer,
    int startFrame,
    int endFrame,
    CalibrationType visibleCalType,
    CalibrationType thermalCalType
  ) throws IOException;

  ////////////////////////////////////////////////////////////

} // RadiometerDataWriter interface

////////////////////////////////////////////////////////////////////////
