////////////////////////////////////////////////////////////////////////
/*
     FILE: EllipsoidModelFactory.java
  PURPOSE: Creates various standard ellipsoid models.
   AUTHOR: Peter Hollemans
     DATE: 2005/10/17
  CHANGES: n/a

  Copyright (C) 2005, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.orbit;

// Imports
// -------
import java.util.*;

/**
 * The <code>EllipsoidModelFactory</code> class creates a number of
 * standard Earth ellipsoid models.
 */
public class EllipsoidModelFactory {

  ////////////////////////////////////////////////////////////

  /** Creates a WGS72 ellipsoid model. */
  public static EllipsoidModel getWGS72 () {

    return (new EllipsoidModel (6378135.0, 6356750.519915));

  } // getWGS72

  ////////////////////////////////////////////////////////////

  /** Creates a WGS84 ellipsoid model. */
  public static EllipsoidModel getWGS84 () {

    return (new EllipsoidModel (6378137.0, 6356752.314245));

  } // getWGS84

  ////////////////////////////////////////////////////////////

} // EllipsoidModelFactory class

////////////////////////////////////////////////////////////////////////
