////////////////////////////////////////////////////////////////////////
/*
     FILE: FY1Spacecraft.java
  PURPOSE: Retrieves CMA FY1 series spacecraft information.
   AUTHOR: Peter Hollemans
     DATE: 2007/03/16
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.orbit;

// Imports
// -------
import java.util.*;
import terrenus.instrument.*;

/**
 * The <code>FY1Spacecraft</code> class is for the China
 * Meteorological Administration (CMA) Fengyun 1 series of spacecrafts
 * with the MVISR sensor package.  Currently MVISR is the only
 * instrument reported by {@link #getInstruments}.
 */
public class FY1Spacecraft 
  extends AbstractSpacecraft {

  // Variables
  // ---------

  /** The list of instruments. */
  private static List<Instrument> instrumentList;

  ////////////////////////////////////////////////////////////

  static {

    // Create list of instruments
    // --------------------------
    instrumentList = new ArrayList<Instrument>();
    instrumentList.add (MVISR.getInstance());

  } // static

  ////////////////////////////////////////////////////////////

  public List<Instrument> getInstruments() { return (instrumentList); }

  ////////////////////////////////////////////////////////////

  /**
   * Creates a new FY1 spacecraft using the NORAD ID.
   *
   * @param noradId the NORAD orbital element ID.
   *
   * @throws RuntimeException if the spacecraft properties cannot be
   * found or parsed.
   */
  public FY1Spacecraft (
    int noradId
  ) {

    super (noradId);

  } // FY1Spacecraft constructor

  ////////////////////////////////////////////////////////////

  /** Tests this class. */
  public static void main (String[] argv) throws Exception {

    AbstractSpacecraft.main (new String[] {"27431"});

  } // main

  ////////////////////////////////////////////////////////////

} // FY1Spacecraft class

////////////////////////////////////////////////////////////////////////
