////////////////////////////////////////////////////////////////////////
/*
     FILE: NOAAKLMSpacecraft.java
  PURPOSE: Retrieves NOAA KLMNN' spacecraft information.
   AUTHOR: Peter Hollemans
     DATE: 2007/03/01
  CHANGES: 2010/12/20, PFH, added NOAA-19 to the list of test spacecraft

  Copyright (C) 2007-2010, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.orbit;

// Imports
// -------
import java.util.*;
import terrenus.instrument.*;

/**
 * The <code>NOAAKLMSpacecraft</code> class is for the NOAA-15,
 * -16, -17, -18, and -19 series of spacecrafts with the AVHRR/3
 * sensor package.  Currently AVHRR is the only instrument
 * reported by {@link #getInstruments}.
 */
public class NOAAKLMSpacecraft 
  extends AbstractSpacecraft {

  // Variables
  // ---------

  /** The list of instruments. */
  private static List<Instrument> instrumentList;

  ////////////////////////////////////////////////////////////

  static {

    // Create list of instruments
    // --------------------------
    instrumentList = new ArrayList<Instrument>();
    instrumentList.add (AVHRR.getInstance());

  } // static

  ////////////////////////////////////////////////////////////

  public List<Instrument> getInstruments() { return (instrumentList); }

  ////////////////////////////////////////////////////////////

  /**
   * Creates a new NOAA spacecraft using the NORAD ID.
   *
   * @param noradId the NORAD orbital element ID.
   *
   * @throws RuntimeException if the spacecraft properties cannot be
   * found or parsed.
   */
  public NOAAKLMSpacecraft (
    int noradId
  ) {

    super (noradId);

  } // NOAAKLMSpacecraft constructor

  ////////////////////////////////////////////////////////////

  /** Tests this class. */
  public static void main (String[] argv) throws Exception {

    AbstractSpacecraft.main (new String[] {"25338"});
    AbstractSpacecraft.main (new String[] {"26536"});
    AbstractSpacecraft.main (new String[] {"27453"});
    AbstractSpacecraft.main (new String[] {"28654"});
    AbstractSpacecraft.main (new String[] {"33591"});

  } // main

  ////////////////////////////////////////////////////////////

} // NOAAKLMSpacecraft class

////////////////////////////////////////////////////////////////////////
