////////////////////////////////////////////////////////////////////////
/*
     FILE: OrbitalElementSource.java
  PURPOSE: Provides two-line orbital elements.
   AUTHOR: Peter Hollemans
     DATE: 2005/08/05
  CHANGES: n/a

  Copyright (C) 2005, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.orbit;

// Imports
// -------
import java.util.*;

/**
 * The <code>OrbitalElementSource</code> interface is for
 * providers of two-line orbital elements for the SGP4/SDP4 orbital
 * model.
 */
public interface OrbitalElementSource {

  /**
   * Gets the orbital elements for the specified satellite and date.
   * 
   * @param satid the NORAD orbital element ID for the satellite.
   * @param date the date for orbital element retrieval.
   * @param maxDays the maximum allowed difference between date and orbital
   * element epoch in days.
   *
   * @return the orbital elements as two strings, or null if none could be
   * found within the specified tolerance.
   */
  public String[] getElements (
    int satid,
    Date date,
    double maxDays
  );

} // OrbitalElementSource interface

////////////////////////////////////////////////////////////////////////
