////////////////////////////////////////////////////////////////////////
/*
     FILE: GlobalHRPTStream.java
  PURPOSE: Reads Global Imaging HRPT data.
   AUTHOR: Peter Hollemans
     DATE: 2005/06/19
  CHANGES: 2007/01/28, PFH, added getFormat()
           2007/03/14, PFH, modified to extend HRPTStream

  Copyright (C) 2005, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.telemetry;

// Imports
// -------
import java.io.*;

/**
 * The <code>GlobalHRPTStream</code> class reads Global Imaging NOAA
 * HRPT storage format.  The file structure is as follows:
 * <ul>
 *   <li>240 byte file header</li>
 *   <li>22180 bytes per HRPT minor frame</li>
 *   <li>10-bit words are stored as least significant 10 bits
 *   in each 16-bit word
 * </ul>
 */
public class GlobalHRPTStream 
  extends HRPTStream {

  ////////////////////////////////////////////////////////////

  public String getFormat () { return ("Global Imaging HRPT"); }

  ////////////////////////////////////////////////////////////

  /** Creates a new stream using the specified file as input. */
  public GlobalHRPTStream (
    String fileName
  ) throws IOException {

    initStream (getInputBuffer (fileName), 240, 22180, 0, false);

  } // GlobalHRPTStream constructor

  ////////////////////////////////////////////////////////////

} // GlobalHRPTStream class

////////////////////////////////////////////////////////////////////////
