////////////////////////////////////////////////////////////////////////
/*
     FILE: HRPTAuxSyncChecker.java
  PURPOSE: Checks NOAA HRPT auxiliary sync.
   AUTHOR: Peter Hollemans
     DATE: 2007/02/20
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.telemetry;

// Imports
// -------
import java.nio.*;

/**
 * The <code>HRPTAuxSyncChecker</code> class checks NOAA HRPT
 * auxiliary synchronization data.
 */
public class HRPTAuxSyncChecker 
  extends ShortSignatureChecker {

  // Constants
  // ---------

  /** The 100 expected auxiliary sync words. */
  public static final short[] AUX_SYNC_WORDS = new short[] {
    (short) 0x3e2,
    (short) 0x3f3,
    (short) 0x1b5,
    (short) 0x2bd,
    (short) 0x284,
    (short) 0x115,
    (short) 0x1c4,
    (short) 0x1d3,
    (short) 0x341,
    (short) 0xe0,
    (short) 0x2b6,
    (short) 0x3de,
    (short) 0xdc,
    (short) 0x199,
    (short) 0x3f2,
    (short) 0x193,
    (short) 0x28e,
    (short) 0x69,
    (short) 0x3e,
    (short) 0x363,
    (short) 0x4b,
    (short) 0x95,
    (short) 0x140,
    (short) 0x2d5,
    (short) 0x29c,
    (short) 0x245,
    (short) 0x362,
    (short) 0x6d,
    (short) 0xa6,
    (short) 0x3ad,
    (short) 0x3fe,
    (short) 0x3b,
    (short) 0x3dd,
    (short) 0xb6,
    (short) 0x1cd,
    (short) 0xc5,
    (short) 0x2ef,
    (short) 0x167,
    (short) 0x2c0,
    (short) 0x42,
    (short) 0x183,
    (short) 0xee,
    (short) 0x352,
    (short) 0x2ea,
    (short) 0x1d9,
    (short) 0x23d,
    (short) 0x11a,
    (short) 0x6,
    (short) 0xd4,
    (short) 0xa9,
    (short) 0x26f,
    (short) 0x2f9,
    (short) 0x3d3,
    (short) 0x152,
    (short) 0xf9,
    (short) 0x1c0,
    (short) 0x14b,
    (short) 0x38f,
    (short) 0x355,
    (short) 0x218,
    (short) 0x143,
    (short) 0x2bf,
    (short) 0x2c8,
    (short) 0x172,
    (short) 0x1e,
    (short) 0x384,
    (short) 0x20f,
    (short) 0x3d1,
    (short) 0x11e,
    (short) 0x9e,
    (short) 0x1a,
    (short) 0x31c,
    (short) 0x2c1,
    (short) 0x64,
    (short) 0x1b0,
    (short) 0x203,
    (short) 0x279,
    (short) 0x4d,
    (short) 0x41,
    (short) 0x1e9,
    (short) 0xba,
    (short) 0x65,
    (short) 0x196,
    (short) 0x230,
    (short) 0x94,
    (short) 0x166,
    (short) 0x2e6,
    (short) 0x71,
    (short) 0x36e,
    (short) 0x1c5,
    (short) 0x1f5,
    (short) 0x372,
    (short) 0x20d,
    (short) 0x39d,
    (short) 0x179,
    (short) 0x144,
    (short) 0x24d,
    (short) 0x252,
    (short) 0x1f0,
    (short) 0x3cc
  };

  // Variables
  // ---------

  /** The static instance of this class. */
  private static HRPTAuxSyncChecker instance = new HRPTAuxSyncChecker();

  ////////////////////////////////////////////////////////////

  /** Gets the single instance of this class. */
  public static HRPTAuxSyncChecker getInstance () { return (instance); }

  ////////////////////////////////////////////////////////////

  /** Creates a new frame sync checker. */
  private HRPTAuxSyncChecker () {

    super (AUX_SYNC_WORDS, (short) 0x03ff);

  } // HRPTAuxSyncChecker constructor

  ////////////////////////////////////////////////////////////

} // HRPTAuxSyncChecker class

////////////////////////////////////////////////////////////////////////
