////////////////////////////////////////////////////////////////////////
/*
     FILE: RawCHRPTStream.java
  PURPOSE: Reads raw CHRPT data.
   AUTHOR: Peter Hollemans
     DATE: 2010/12/20
  CHANGES: n/a

  Copyright (C) 2010, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.telemetry;

// Imports
// -------
import java.io.*;

/**
 * The <code>RawCHRPTStream</code> class reads raw CMA
 * CHRPT storage format.  The file structure is as follows:
 * <ul>
 *   <li>44360 bytes per CHRPT minor frame</li>
 *   <li>10-bit words are stored as least significant 10 bits
 *   in each 16-bit word
 * </ul>
 */
public class RawCHRPTStream 
  extends CHRPTStream {

  ////////////////////////////////////////////////////////////

  public String getFormat () { return ("Raw CMA CHRPT"); }

  ////////////////////////////////////////////////////////////

  /** Creates a new stream using the specified file as input. */
  public RawCHRPTStream (
    String fileName
  ) throws IOException {

    initStream (getInputBuffer (fileName), 0, 44360, 0, false);

  } // RawCHRPTStream constructor

  ////////////////////////////////////////////////////////////

  public int getDayCount (int frame) throws IOException {
  
    // TODO: We return the day count here set for the stream and hope
    // that the decoder is smart enough to handle a day change
    // in the middle of a pass.  There should be a better algorithm
    // for this.

    return (getDayCount());
  
  } // getDayCount

  ////////////////////////////////////////////////////////////

} // RawCHRPTStream class

////////////////////////////////////////////////////////////////////////
