////////////////////////////////////////////////////////////////////////
/*
     FILE: ShortSignatureChecker.java
  PURPOSE: Checks signatures of short values.
   AUTHOR: Peter Hollemans
     DATE: 2007/02/20
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.telemetry;

// Imports
// -------
import java.nio.*;

/**
 * The <code>ShortSignatureChecker</code> class checks signatures of
 * short (16-bit) values.
 */
public class ShortSignatureChecker 
  implements SignatureChecker {

  // Variables
  // ---------

  /** The known signature pattern. */
  private short[] signature;

  /** The mask for value checking. */
  private short mask;

  ////////////////////////////////////////////////////////////

  /** 
   * Creates a new checker.
   *
   * @param signature the signature pattern data.
   * @param mask the mask value.  Only the bits turned on in the
   * mask are used for comparing short values.
   */
  public ShortSignatureChecker (
    short[] signature,
    short mask
  ) {

    this.signature = signature;
    this.mask = mask;

  } // ShortSignatureChecker constructor

  ////////////////////////////////////////////////////////////

  public int getDifferenceCount (
    ByteBuffer buf
  ) {

    // Loop over each short word
    // -------------------------
    int errors = 0;
    ShortBuffer shortBuf = buf.asShortBuffer();
    for (int i = 0; i < signature.length; i++) {

      // Check if any bits differ
      // ------------------------
      short word = shortBuf.get (i);
      short xor = (short) ((word ^ signature[i]) & mask);

      // Count bits different
      // --------------------
      if (xor != 0) {
        for (int bit = 0; bit < 16; bit++) {
          if (((0x1 << bit) & xor) != 0) errors++;
        } // for
      } // if

    } // for

    return (errors);

  } // getDifferenceCount

  ////////////////////////////////////////////////////////////

} // ShortSignatureChecker class

////////////////////////////////////////////////////////////////////////
